<?php

namespace Assetdocs\Department\Http\Controllers;

use App\Http\Controllers\Controller as BaseController;
use Assetdocs\Department\Http\Requests\DepartmentUserApiRequest;
use Assetdocs\Department\Interfaces\DepartmentRepositoryInterface;
use Assetdocs\Department\Models\Department;

/**
 * User API controller class.
 */
class DepartmentUserApiController extends BaseController
{

    /**
     * The authentication guard that should be used.
     *
     * @var string
     */
    protected $guard = 'api';

    /**
     * Initialize department controller.
     *
     * @param type DepartmentRepositoryInterface $department
     *
     * @return type
     */
    public function __construct(DepartmentRepositoryInterface $department)
    {
        $this->middleware('api');
        $this->middleware('jwt.auth:api');
        $this->setupTheme(config('theme.themes.user.theme'), config('theme.themes.user.layout'));

        $this->repository = $department;
        parent::__construct();
    }

    /**
     * Display a list of department.
     *
     * @return json
     */
    public function index(DepartmentUserApiRequest $request)
    {
        $departments  = $this->repository
            ->pushCriteria(new \Lavalite\Department\Repositories\Criteria\DepartmentUserCriteria())
            ->setPresenter('\\Assetdocs\\Department\\Repositories\\Presenter\\DepartmentListPresenter')
            ->scopeQuery(function($query){
                return $query->orderBy('id','DESC');
            })->all();
        $departments['code'] = 2000;
        return response()->json($departments) 
            ->setStatusCode(200, 'INDEX_SUCCESS');

    }

    /**
     * Display department.
     *
     * @param Request $request
     * @param Model   Department
     *
     * @return Json
     */
    public function show(DepartmentUserApiRequest $request, Department $department)
    {

        if ($department->exists) {
            $department         = $department->presenter();
            $department['code'] = 2001;
            return response()->json($department)
                ->setStatusCode(200, 'SHOW_SUCCESS');;
        } else {
            return response()->json([])
                ->setStatusCode(400, 'SHOW_ERROR');
        }

    }

    /**
     * Show the form for creating a new department.
     *
     * @param Request $request
     *
     * @return json
     */
    public function create(DepartmentUserApiRequest $request, Department $department)
    {
        $department         = $department->presenter();
        $department['code'] = 2002;
        return response()->json($department)
            ->setStatusCode(200, 'CREATE_SUCCESS');
    }

    /**
     * Create new department.
     *
     * @param Request $request
     *
     * @return json
     */
    public function store(DepartmentUserApiRequest $request)
    {
        try {
            $attributes             = $request->all();
            $attributes['user_id']  = user_id('admin.api');
            $department          = $this->repository->create($attributes);
            $department          = $department->presenter();
            $department['code']  = 2004;

            return response()->json($department)
                ->setStatusCode(201, 'STORE_SUCCESS');
        } catch (Exception $e) {
            return response()->json([
                'message' => $e->getMessage(),
                'code'    => 4004,
            ])->setStatusCode(400, 'STORE_ERROR');
        }

    }

    /**
     * Show department for editing.
     *
     * @param Request $request
     * @param Model   $department
     *
     * @return json
     */
    public function edit(DepartmentUserApiRequest $request, Department $department)
    {
        if ($department->exists) {
            $department         = $department->presenter();
            $department['code'] = 2003;
            return response()-> json($department)
                ->setStatusCode(200, 'EDIT_SUCCESS');;
        } else {
            return response()->json([])
                ->setStatusCode(400, 'SHOW_ERROR');
        }
    }

    /**
     * Update the department.
     *
     * @param Request $request
     * @param Model   $department
     *
     * @return json
     */
    public function update(DepartmentUserApiRequest $request, Department $department)
    {
        try {

            $attributes = $request->all();

            $department->update($attributes);
            $department         = $department->presenter();
            $department['code'] = 2005;

            return response()->json($department)
                ->setStatusCode(201, 'UPDATE_SUCCESS');


        } catch (Exception $e) {

            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 4005,
            ])->setStatusCode(400, 'UPDATE_ERROR');

        }
    }

    /**
     * Remove the department.
     *
     * @param Request $request
     * @param Model   $department
     *
     * @return json
     */
    public function destroy(DepartmentUserApiRequest $request, Department $department)
    {

        try {

            $t = $department->delete();

            return response()->json([
                'message'  => trans('messages.success.delete', ['Module' => trans('department::department.name')]),
                'code'     => 2006
            ])->setStatusCode(202, 'DESTROY_SUCCESS');

        } catch (Exception $e) {

            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 4006,
            ])->setStatusCode(400, 'DESTROY_ERROR');
        }
    }
}
